################################################################
# Data Analysis Code - Lightweight Relief Shearing for Enhanced Terrain Perception on Interactive Maps (CHI '15)
# Performs data processing and generates plots for Experiment 1
# 2014 Wesley Willett
################################################################

# The Directory containing all R scripts and data files.
# SET THIS! --- Should point to the current absolute path to the directory containing the R scripts and data files.
sourceDir <- "YOUR_SOURCE_DIRECTORY"

setwd(sourceDir)
source("bootstrap_macros.R")
source("plotting.R")
library(PropCIs)

#############################################################################

# Load raw data file
files <- "log-experiment1.csv"
datasetName <- "Experiment 1"


score_formatter <- function(x) {
    label <- sprintf('%.2f', x)
    return (label)
}

#############################################################################
# Build the columns of the cleaned up table

c_participant <- NULL
c_expertise <- NULL
c_tech <- NULL
c_map <- NULL
c_correct <- NULL
c_distance <- NULL
c_elevation <- NULL
c_trialnumber <- NULL
c_cumerrors <- NULL
c_time <- NULL

for (file in files) {
	rawdata <- read.table(file,header=TRUE,sep=",")

	# Loop through the raw data and, for each row, add values to each of the columns
	for(i in 1:nrow(rawdata)) {
    row <- rawdata[i,]
    
    # Participant Id
    participant <- as.character(row$Participant.Id)
	
    # Skip rows that don't contain a red/blue response (e.g. pauses/skips)
    if (row$Response != "red" && row$Response != "blue"){ next }
		
		# Participant filter - keep only the participants for Experiment 1	
		if (participant < 10 || participant > 30) { next }
		
		# Expertise - Assign each participant the right expertise value ("E" for experts, "N" for non-experts)
		expertise <- switch(participant,
									"10" = "E", "11" = "E", "13" = "E", "14" = "E", "17" = "E", "25" = "E", "26" = "E", "27" = "E", "28" = "E", "29" = "E", 
									"12" = "N", "15" = "N", "16" = "N", "18" = "N", "19" = "N", "20" = "N", "21" = "N", "22" = "N", "23" = "N", "24" = "N", "U") 
			
    # Assign Technique Label
    tech <- "Static"
    if (row$Animation...Enabled == "true") {tech <- "Animated"}
    if (row$Pan.Enabled == "true") {tech <- "Interactive"}
    
    # Map
    filename <- row$Grid.Source
    filenameregex <- ".*/([^/]*)\\.grd$"
    map <- gsub(filenameregex, "\\1", filename, perl = TRUE)
    if (map == "Blanca-Training"){ next } #Ignore training map

		# Elevation and Distance
		elevation <- row$Abs.Elevation.Diff
		distance <- row$Abs.DistanceXY	
		
		# Trial Number and Cumulative Errors
		trialnumber <- row$Trial.Number
		cumerrors <- row$Cumulative.Errors
	   
    # Determine whether the result was correct
    correct <- ifelse((row$RedZ > row$BlueZ && row$Response == "red") || (row$BlueZ > row$RedZ && row$Response == "blue"), 1, 0)
    time <- row$Task.Time
    
    # Update columns
    c_participant <- append(c_participant, participant)
    c_expertise <- append(c_expertise, expertise)
    c_tech <- append(c_tech, tech)
    c_map <- append(c_map, map)
    c_correct <- append(c_correct, correct)
    c_elevation <- append(c_elevation, elevation)
    c_distance <- append(c_distance, distance)
    c_trialnumber <- append(c_trialnumber, trialnumber)
    c_cumerrors <- append(c_cumerrors, cumerrors)
    c_time <- append(c_time, time)
	}
}

# Use columns to build a cleaned-up data frame
data <- data.frame(
	factor(c_participant),
	factor(c_expertise, levels = c("E","N","U"), labels = c("Expert", "Non-Expt.", "Unknown")),
	factor(c_map),
	factor(c_tech, levels = c("Static", "Animated", "Interactive")), # reorder factors
	factor(c_correct),
	c_elevation,
	c_distance,
	c_trialnumber,
	c_cumerrors,
	c_time)

# Assign column names
colnames(data) <- c("participant", "expertise", "map", "tech", "correct", "elevation", "distance", "trialnumber", "cumerrors", "time")


#############################################################################
# COMPARING TECHNIQUES
# Compare each participant's accuracy rate across each of the techniques

# Create columns
c_tech <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# Compute each participant's accuracy for each technique regardless of map
for (tech in levels(data$tech)) {
	scores <- NULL
	for (participant in levels(data$participant)) {
		success <- data$correct[data$participant == participant & data$tech == tech]
		#score <- length(success[success ==1]) / length(success)  # Computes crude %correct
		score <- (percentCI(success, 1)[1] - 50) / 50 # Computes a CI, takes the mean, then normalizes
		scores <- append(scores, score)
	}
	mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
	c_map <- append(c_map, "All maps")
	c_tech <- append(c_tech, tech)
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}

# Build data frame for plotting
allScoresAnalysis <- data.frame(
	factor(c_map),
	factor(c_tech, levels = c("Static", "Interactive", "Animated"), labels = c("Static", "Integrated Shearing", "Animated Shearing")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(allScoresAnalysis) <- c("map", "tech", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(allScoresAnalysis, "Accuracy", datasetName, "tech", 0, 1, score_formatter, height=1)

#############################################################################
# COMPARE TECHNIQUE PAIRS

c_techs <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# By  technique
for (t1 in seq(length(levels(data$tech)))) {
	tech1 <- levels(data$tech)[t1]
	for (t2 in seq(t1, length(levels(data$tech)))) {
		tech2 <- levels(data$tech)[t2]
		if (tech1 == tech2 ){ next }
		scores <- NULL
		for (participant in levels(data$participant)) {
			results1 <- data$correct[data$participant == participant & data$tech == tech1]
			results2 <- data$correct[data$participant == participant & data$tech == tech2]
			correct1 <- (percentCI(results1, 1)[1] - 50) / 50
			correct2 <- (percentCI(results2, 1)[1] - 50) / 50
			score <- correct2 - correct1
			scores <- append(scores, score)
		}
		meanci <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
		c_techs <- append(c_techs, paste(tech2,"-",tech1))
		c_mean <- append(c_mean, meanci[1])
		c_cimin <- append(c_cimin, meanci[2])
		c_cimax <- append(c_cimax, meanci[3])
	}
}

techPairsAnalysis <- data.frame(
	factor(c_techs, levels = c("Interactive - Animated","Interactive - Static","Animated - Static"),
						  labels = c("Integrated - Animated","Integrated - Static","Animated - Static")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(techPairsAnalysis) <- c("techs", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(techPairsAnalysis, "Improvement in Accuracy", datasetName, "techs", -0.1, 0.25, score_formatter, height=1)


#############################################################################
# COMPARE MAPS

c_tech <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# Compare techniques across all maps
for (map in levels(data$map)) {
	scores <- NULL
	for (participant in levels(data$participant)) {
		success <- data$correct[data$participant == participant & data$map == map]
		score <- (percentCI(success, 1)[1] - 50) / 50 # Computes a CI, takes the mean, then normalizes
		scores <- append(scores, score)
	}
	mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
	c_map <- append(c_map, map)
	c_tech <- append(c_tech, "All Techniques")
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}

allScoresAnalysis <- data.frame(
	factor(c_map, labels=c("Swiss Natl. Map", "US Natl. Park Map", "Google Map SF")),
	factor(c_tech),
	c_mean, c_cimin, c_cimax)
colnames(allScoresAnalysis) <- c("map", "tech", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(allScoresAnalysis, "Accuracy", datasetName, "map", 0, 1, score_formatter, height=1)

#############################################################################
# COMPARE EXPERTISE

c_expertise <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# Compare techniques across all maps
for (expertise in levels(data$expertise)) {
	scores <- NULL
	for (participant in levels(data$participant)) {
		success <- data$correct[data$participant == participant & data$expertise == expertise]
		if (length(success) == 0) { next }
		score <- (percentCI(success, 1)[1] - 50) / 50 # Computes a CI, takes the mean, then normalizes
		scores <- append(scores, score)
	}
	if (length(scores) == 0) { next }
	mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
	c_expertise <- append(c_expertise, expertise)
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}

expertiseAnalysis <- data.frame(
	factor(c_expertise),
	c_mean, c_cimin, c_cimax)
colnames(expertiseAnalysis) <- c("expertise", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(expertiseAnalysis, "Accuracy", datasetName, "expertise", 0, 1, score_formatter, height=1)


#############################################################################
# COMPARE INTERACTION BETWEEN EXPERTISE AND MAP

c_expertise <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# Compare techniques across all maps
for (expertise in levels(data$expertise)) {
	for (map in levels(data$map)) {
		scores <- NULL
		for (participant in levels(data$participant)) {
			success <- data$correct[data$participant == participant & data$expertise == expertise & data$map == map]
			if (length(success) == 0) { next }
			score <- (percentCI(success, 1)[1] - 50) / 50 # Computes a CI, takes the mean, then normalizes
			scores <- append(scores, score)
		}
		if (length(scores) == 0) { next }
		mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
		c_map <- append(c_map, map)
		c_expertise <- append(c_expertise, expertise)
		c_mean <- append(c_mean, mean[1])
		c_cimin <- append(c_cimin, mean[2])
		c_cimax <- append(c_cimax, mean[3])
	}
}

expertiseMapAnalysis <- data.frame(
	factor(c_map, labels=c("Swiss Topo Map", "US NPS Map", "Google Map")),
	factor(c_expertise),
	c_mean, c_cimin, c_cimax)
colnames(expertiseMapAnalysis) <- c("map", "expertise", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(expertiseMapAnalysis, "Accuracy", datasetName, "expertise", 0, 1, score_formatter, height=1, facets=(. ~ map))

#############################################################################
# COMPARE INTERACTION BETWEEN EXPERTISE AND TECHNIQUE

c_expertise <- NULL
c_map <- NULL
c_techs <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

for (expertise in levels(data$expertise)) {
	for (t1 in seq(length(levels(data$tech)))) {
		tech1 <- levels(data$tech)[t1]
		for (t2 in seq(t1, length(levels(data$tech)))) {
			tech2 <- levels(data$tech)[t2]
			if (tech1 == tech2 ){ next }
			scores <- NULL
			for (participant in levels(data$participant)) {
				results1 <- data$correct[data$expertise == expertise & data$participant == participant & data$tech == tech1]
				results2 <- data$correct[data$expertise == expertise & data$participant == participant & data$tech == tech2]
				if (length(results1) == 0 || length(results2) == 0) { next }
				correct1 <- (percentCI(results1, 1)[1] - 50) / 50
				correct2 <- (percentCI(results2, 1)[1] - 50) / 50
				score <- correct2 - correct1
				scores <- append(scores, score)
			}
			if (length(scores) == 0) { next }
			mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
			c_techs <- append(c_techs, paste(tech2,"-",tech1))
			c_map <- append(c_map, "All Maps")
			c_expertise <- append(c_expertise, expertise)
			c_mean <- append(c_mean, mean[1])
			c_cimin <- append(c_cimin, mean[2])
			c_cimax <- append(c_cimax, mean[3])
		}
	}
}

expertiseTechDiffAnalysis <- data.frame(
	factor(c_techs, levels = c("Interactive - Animated","Interactive - Static","Animated - Static"),
						  labels = c("Integrated - Animated","Integrated - Static","Animated - Static")), # reorder factors
	factor(c_map),
	factor(c_expertise),
	c_mean, c_cimin, c_cimax)
colnames(expertiseTechDiffAnalysis) <- c("techs", "map", "expertise", "pointEstimate", "ci.min", "ci.max")

# plot
hplot(expertiseTechDiffAnalysis, "Improvement in Accuracy", datasetName, "techs", -0.1, 0.3, score_formatter, height=1.5, facets=(expertise ~ .))


#############################################################################
# COMPARE INTERACTION BETWEEN EXPERTISE, MAP AND TECHNIQUE

c_expertise <- NULL
c_map <- NULL
c_techs <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

for (expertise in levels(data$expertise)) {
	for(map in levels(data$map)) {
		for (t1 in seq(length(levels(data$tech)))) {
			tech1 <- levels(data$tech)[t1]
			for (t2 in seq(t1, length(levels(data$tech)))) {
				tech2 <- levels(data$tech)[t2]
				if (tech1 == tech2 ){ next }
				scores <- NULL
				for (participant in levels(data$participant)) {
					results1 <- data$correct[data$expertise == expertise & data$participant == participant & data$tech == tech1 & data$map == map]
					results2 <- data$correct[data$expertise == expertise & data$participant == participant & data$tech == tech2 & data$map == map]
					if (length(results1) == 0 || length(results2) == 0) { next }
					correct1 <- (percentCI(results1, 1)[1] - 50) / 50
					correct2 <- (percentCI(results2, 1)[1] - 50) / 50
					score <- correct2 - correct1
					scores <- append(scores, score)
				}
				if (length(scores) == 0) { next }
				mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
				c_techs <- append(c_techs, paste(tech2,"-",tech1))
				c_map <- append(c_map, map)
				c_expertise <- append(c_expertise, expertise)
				c_mean <- append(c_mean, mean[1])
				c_cimin <- append(c_cimin, mean[2])
				c_cimax <- append(c_cimax, mean[3])
			}
		}
	}
}

expertiseMapTechDiffAnalysis <- data.frame(
	factor(c_techs),
	factor(c_map),
	factor(c_expertise),
	c_mean, c_cimin, c_cimax)
colnames(expertiseMapTechDiffAnalysis) <- c("techs", "map", "expertise", "pointEstimate", "ci.min", "ci.max")

# plot
plotResults(expertiseMapTechDiffAnalysis, "Mean accuracy", datasetName, -0.3, 0.4, score_formatter, x="techs", facets=(. ~ expertise + map))


#############################################################################
# LEARNING EFFECTS
# ---- Compute accuracy for every trial block ----

c_trial <- NULL
c_trialcnt <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

num_trials <- 15
tasks_before_first <- 50
tasks_per_trial <-20

for (trial in seq(1,num_trials)) {
	scores <- subset(data, trialnumber > tasks_before_first + (trial - 1) * tasks_per_trial & 
							   trialnumber < tasks_before_first + trial * tasks_per_trial)$correct
	mean <- (percentCI(scores, 1)[1] - 50) / 50 
	c_trial <- append(c_trial, trial)
	c_trialcnt <- append(c_trialcnt, length(c_trial))
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}
learningByTrialsAnalysis <- data.frame(
	factor(c_trial),
	c_trialcnt,
	c_mean, c_cimin, c_cimax)
colnames(learningByTrialsAnalysis) <- c("trial", "trialcnt", "pointEstimate", "ci.min", "ci.max")

# Compute Regression
lm(pointEstimate ~ trialcnt, learningByTrialsAnalysis)
fit <- lm(pointEstimate ~ trialcnt, learningByTrialsAnalysis)
confint(fit)

# Summary plots (not really necessary)
#fit <- lm(pointEstimate ~ trialcnt, learningByTrialsAnalysis)
#summary(fit) # show results
#layout(matrix(c(1,2,3,4),2,2)) # optional 4 graphs/page 
#plot(fit)

# ---- Compute accuracy for every trial block for each participant ---- 
c_trial <- NULL
c_trialcnt <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL
num_trials <- 15
tasks_before_first <- 50
tasks_per_trial <-20

for (trial in seq(1,num_trials)) {
	scores <- NULL
	for (part in levels(data$participant)) {
		trialblock = subset(data, participant == part & trialnumber > tasks_before_first + (trial - 1) * tasks_per_trial &
																trialnumber < tasks_before_first + trial * tasks_per_trial)
		success <- trialblock$correct
		score <- (percentCI(success, 1)[1] - 50) / 50
		scores <- append(scores, score)
	}
	mean <- bootstrapMeanCI(scores)
	c_trial <- append(c_trial, trial)
	c_trialcnt <- append(c_trialcnt, length(c_trial))
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}
learningByParTrialsAnalysis <- data.frame(
	factor(c_trial),
	c_trialcnt,
	c_mean, c_cimin, c_cimax)
colnames(learningByParTrialsAnalysis) <- c("trial", "trialcnt", "pointEstimate", "ci.min", "ci.max")

# Compute Regression
lm(pointEstimate ~ trialcnt, learningByParTrialsAnalysis)
fit <- lm(pointEstimate ~ trialcnt, learningByParTrialsAnalysis)
confint(fit)

# Summary plots (not really necessary)
#summary(fit) # show results
#layout(matrix(c(1,2,3,4),2,2)) # optional 4 graphs/page 
#plot(fit)


#############################################################################
# TIME

c_tech <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# All maps 
for (tech in levels(data$tech)) {
	times <- data$time[data$tech == tech]
	mean <- exactMeanCI(times, transformation = inverseTransform)
	c_map <- append(c_map, "All maps")
	c_tech <- append(c_tech, tech)
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}
# Maps by technique
for (map in levels(data$map)) {
	for (tech in levels(data$tech)) {
		times <- data$time[data$map == map & data$tech == tech]
		mean <- exactMeanCI(times, transformation = inverseTransform)
		c_map <- append(c_map, map)
		c_tech <- append(c_tech, tech)
		c_mean <- append(c_mean, mean[1])
		c_cimin <- append(c_cimin, mean[2])
		c_cimax <- append(c_cimax, mean[3])
	}
}

timeAnalysis <- data.frame(
	factor(c_map),
	factor(c_tech, , levels = c("Static", "Animated", "Interactive")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(timeAnalysis) <- c("map", "tech", "pointEstimate", "ci.min", "ci.max")

# plot
second_formatter <- function(x) {
    label <- sprintf('%d s', x)
    return (label)
}
hplot(timeAnalysis, "Imrovement in Accuracy", datasetName, "tech", 0, 4.2, second_formatter, height=4, facets=(map ~ .))



#############################################################################
# INTERACTION BETWEEN MAP/TECHNIQUE
# Analyze percent correct trials per map and per technique.

c_techs <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# By map and technique
for (map in levels(data$map)) {
	for (t1 in seq(length(levels(data$tech)))) {
		tech1 <- levels(data$tech)[t1]
		for (t2 in seq(t1, length(levels(data$tech)))) {
			tech2 <- levels(data$tech)[t2]
			if (tech1 == tech2 ){ next }
			scores <- NULL
			for (part in levels(data$participant)) {
				results1 <- data$correct[data$map == map & data$participant == part & data$tech == tech1]
				results2 <- data$correct[data$map == map & data$participant == part & data$tech == tech2]
				correct1 <- (percentCI(results1, 1)[1] - 50) / 50
				correct2 <- (percentCI(results2, 1)[1] - 50) / 50
				score <- correct2 - correct1
				scores <- append(scores, score)
			}
			meanci <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
			c_map <- append(c_map, map)
			c_techs <- append(c_techs, paste(tech2,"—",tech1))
			c_mean <- append(c_mean, meanci[1])
			c_cimin <- append(c_cimin, meanci[2])
			c_cimax <- append(c_cimax, meanci[3])
		}
	}
}

techMapInteractionAnalysis <- data.frame(
	factor(c_map),
	factor(c_techs, levels = c("Interactive — Animated","Animated — Static","Interactive — Static")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(techMapInteractionAnalysis) <- c("map", "tech", "pointEstimate", "ci.min", "ci.max")

# plot
plotResults(techMapInteractionAnalysis, "Mean accuracy", datasetName, -0.25, 0.3, score_formatter)


#############################################################################
# INTERACTION BETWEEN TECHNIQUE AND ELEVATION DIFF

c_tech <- NULL
c_elevBin <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

data$elevBin <- cut(data$elevation,5)

# By technique and elevation
for (tech in levels(data$tech)) {
	for (elevBin in levels(data$elevBin)) {
		results <- data$correct[data$tech == tech & data$elevBin == elevBin]
		c_tech <- append(c_tech, tech)
		c_elevBin <- append(c_elevBin, elevBin)
		scores <- NULL
		for (part in levels(data$participant)) {
			results <- data$correct[data$participant == part & data$tech == tech & data$elevBin == elevBin]
			score <- (percentCI(results, 1)[1] - 50) / 50
			scores <- append(scores, score)
		}
		meanci <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
		c_mean <- append(c_mean, meanci[1])
		c_cimin <- append(c_cimin, meanci[2])
		c_cimax <- append(c_cimax, meanci[3])
	}
}

techElevationAnalysis <- data.frame(
	factor(c_elevBin),
	factor(c_tech),
	c_mean, c_cimin, c_cimax)
colnames(techElevationAnalysis) <- c("elevBin","tech", "pointEstimate", "ci.min", "ci.max")

# plot
plotResults(techElevationAnalysis, "Mean accuracy", datasetName, 0, 1, score_formatter, facets=(. ~ elevBin))





